
#include "Graphics.hpp"

void N_Graphics::die_with_sdl_error(std::string message) {
	std::cout << message << ": " << SDL_GetError() << std::endl;
	exit(EXIT_FAILURE);
}

void N_Graphics::die_if_opengl_error() {
	GLenum error = glGetError();
	if (error != GL_NO_ERROR) {
		std::cout << "OpenGL error: " << gluErrorString(error) << std::endl;
		exit(EXIT_FAILURE);
	}
}

SDL_Window* N_Graphics::createWindow(std::string title, int width, int height) {
	SDL_GL_SetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, 2);
	SDL_GL_SetAttribute(SDL_GL_CONTEXT_MINOR_VERSION, 1);
	SDL_Window *window =
		SDL_CreateWindow(title.c_str(),
			SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED,
			width, height, SDL_WINDOW_OPENGL | SDL_WINDOW_SHOWN);
	if (window == NULL)
		die_with_sdl_error("Failed to create window");
	SDL_GLContext context = SDL_GL_CreateContext(window);
	if (context == NULL)
		die_with_sdl_error("Failed to create OpenGL context");
	SDL_GL_SetSwapInterval(1);
#ifndef __APPLE__
	glewExperimental = GL_TRUE;
	glewInit();
#endif
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_NORMALIZE);
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
	die_if_opengl_error();
	return window;
}

void N_Graphics::destroyWindow(SDL_Window *window) {
	SDL_DestroyWindow(window);
}

VertexBuffer N_Graphics::allocateVertexBuffer(int size) {
	GLuint buffer;
	glGenBuffers(1, &buffer);
	glBindBuffer(GL_ARRAY_BUFFER, buffer);
	glBufferData(GL_ARRAY_BUFFER, size, NULL, GL_STATIC_DRAW);
	die_if_opengl_error();
	return buffer;
}

ElementBuffer N_Graphics::allocateElementBuffer(int size) {
	GLuint buffer;
	glGenBuffers(1, &buffer);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, buffer);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, size, NULL, GL_STATIC_DRAW);
	die_if_opengl_error();
	return buffer;
}

void N_Graphics::copyVertexData(VertexBuffer buffer, void *data, int size) {
	glBindBuffer(GL_ARRAY_BUFFER, buffer);
	glBufferSubData(GL_ARRAY_BUFFER, 0, size, data);
	die_if_opengl_error();
}

void N_Graphics::copyElementData(ElementBuffer buffer, void *data, int size) {
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, buffer);
	glBufferSubData(GL_ELEMENT_ARRAY_BUFFER, 0, size, data);
	die_if_opengl_error();
}

void N_Graphics::setVertexArray(VertexBuffer buffer) {
	glEnableClientState(GL_VERTEX_ARRAY);
	glBindBuffer(GL_ARRAY_BUFFER, buffer);
	glVertexPointer(3, GL_FLOAT, 0, 0);
	die_if_opengl_error();
}

void N_Graphics::setColorArray(VertexBuffer buffer) {
	glEnableClientState(GL_COLOR_ARRAY);
	glBindBuffer(GL_ARRAY_BUFFER, buffer);
	glColorPointer(3, GL_FLOAT, 0, 0);
	die_if_opengl_error();
}

void N_Graphics::setNormalArray(VertexBuffer buffer) {
	glEnableClientState(GL_NORMAL_ARRAY);
	glBindBuffer(GL_ARRAY_BUFFER, buffer);
	glNormalPointer(GL_FLOAT, 0, 0);
	die_if_opengl_error();
}

void N_Graphics::setTexCoordArray(VertexBuffer buffer) {
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glBindBuffer(GL_ARRAY_BUFFER, buffer);
	glTexCoordPointer(2, GL_FLOAT, 0, 0);
	die_if_opengl_error();
}

void N_Graphics::drawElements(GLenum mode, ElementBuffer buffer, int count) {
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, buffer);
	glDrawElements(mode, count, GL_UNSIGNED_INT, 0);
	die_if_opengl_error();
}

Texture N_Graphics::loadTexture(std::string bmpFile) {
	SDL_Surface *surface = SDL_LoadBMP(bmpFile.c_str());
	if (surface == NULL)
		die_with_sdl_error("Failed to load image");
	SDL_LockSurface(surface);
	GLuint texture;
	glGenTextures(1, &texture);
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, surface->w, surface->h, 0, GL_BGR,
		GL_UNSIGNED_BYTE, surface->pixels);
	glGenerateMipmap(GL_TEXTURE_2D);
	SDL_FreeSurface(surface);
	die_if_opengl_error();
	return texture;
}

void N_Graphics::setTexture(Texture texture) {
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, texture);
	die_if_opengl_error();
}

void N_Graphics::unsetTexture() {
	glDisable(GL_TEXTURE_2D);
}
