
#include "SDL_GLM_INCLUDES.hpp"
#include "InputHandle.hpp"

namespace gar3d {

	InputHandle::InputHandle() {
		previousKeyboardState = new Uint8[SDL_NUM_SCANCODES];
		currentKeyboardState = new Uint8[SDL_NUM_SCANCODES];
		for (int i = 0; i < SDL_NUM_SCANCODES; i++) {
			previousKeyboardState[i] = 0;
			currentKeyboardState[i] = 0;
		}
	}

	InputHandle::~InputHandle() {
		delete[] previousKeyboardState;
		delete[] currentKeyboardState;
	}

	bool InputHandle::mouseButtonHeld(mouseButton button) const {
		return currentMouseButtonState[button];
	}

	bool InputHandle::mouseButtonReleased(mouseButton button) const {
		return previousMouseButtonState[button]
			&& !currentMouseButtonState[button];
	}

	bool InputHandle::mouseButtonPressed(mouseButton button) const {
		return currentMouseButtonState[button]
			&& !previousMouseButtonState[button];
	}

	bool InputHandle::buttonHeld(Key code) const {
		return currentKeyboardState[code];
	}

	bool InputHandle::buttonReleased(Key code) const {
		return previousKeyboardState[code] && !currentKeyboardState[code];
	}

	bool InputHandle::buttonPressed(Key code) const {
		return currentKeyboardState[code] && !previousKeyboardState[code];
	}

	void InputHandle::mousePosition(int& x, int& y) const {
		int xx, yy;
		SDL_GetMouseState(&xx, &yy);
		x = xx;
		y = yy;
	}

	int InputHandle::mouseX() const {
		int x;
		SDL_GetMouseState(&x, NULL);
		return x;
	};

	int InputHandle::mouseY() const {
		int y;
		SDL_GetMouseState(NULL, &y);
		return y;
	}

	void InputHandle::updateState() {
		// keyboard
		const Uint8* state = SDL_GetKeyboardState(NULL);
		for (int i = 0; i < SDL_NUM_SCANCODES; i++) {
			previousKeyboardState[i] = currentKeyboardState[i];
			currentKeyboardState[i] = state[i];
		}
		if (currentKeyboardState[quitkey]) {
			userQuit = true;
		}
		//mouse
		mouseMotion = glm::vec2(0, 0);
		for (int i = 1; i < 6; i++) {
			previousMouseButtonState[i] = currentMouseButtonState[i];
			currentMouseButtonState[i] = SDL_GetMouseState(NULL, NULL) & SDL_BUTTON(i);
		}
		pollEvents();
	}
	
	void InputHandle::pollEvents(){
		// event handling
		SDL_Event event;
		while (SDL_PollEvent(&event)) {
			switch (event.type) {
			case SDL_QUIT:
				userQuit = true;
				break;
			case SDL_KEYDOWN:
				onKeyDown(event.key);
				break;
			case SDL_KEYUP:
				onKeyUp(event.key);
				break;
			case SDL_MOUSEMOTION:
				onMouseMotion(event.motion);
				break;
			case SDL_MOUSEBUTTONDOWN:
				onMouseButtonDown(event.button);
				break;
			case SDL_MOUSEBUTTONUP:
				onMouseButtonUp(event.button);
				break;
			}
		}
	}

	void InputHandle::onMouseMotion(SDL_MouseMotionEvent& e) {
		mouseMotion.x = e.xrel;
		mouseMotion.y = e.yrel;
	}
	
	void InputHandle::onKeyDown(SDL_KeyboardEvent& e) {
		currentKeyboardState[e.keysym.scancode] = true;
	}
	
	void InputHandle::onKeyUp(SDL_KeyboardEvent& e) {
		currentKeyboardState[e.keysym.scancode] = false;
	}
	
	void InputHandle::onMouseButtonDown(SDL_MouseButtonEvent& e) {
		currentMouseButtonState[e.button] = true;
	}
	
	void InputHandle::onMouseButtonUp(SDL_MouseButtonEvent& e) {
		currentMouseButtonState[e.button] = false;
	}

}  // namespace gar3d
